/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.api.topology;

import de.tud.kom.p2psim.api.common.Position;
import de.tud.kom.p2psim.api.common.SimHostComponent;
import de.tud.kom.p2psim.api.energy.EnergyComponent;
import de.tud.kom.p2psim.api.topology.movement.MovementListener;
import de.tud.kom.p2psim.api.topology.movement.MovementSupported;
import de.tud.kom.p2psim.api.topology.views.TopologyView;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.LocationSensor;
import de.tudarmstadt.maki.simonstrator.api.component.topology.UnderlayTopologyProvider;

/**
 * One component for each host, holding the position. These are maintained
 * inside a {@link Topology} which exists globally. They also include a
 * reference to the topology.
 * 
 * @author Bjoern Richerzhagen
 * @version 1.0, 21.02.2012
 */
public interface TopologyComponent extends SimHostComponent, MovementSupported,
		MovementListener, UnderlayTopologyProvider, LocationSensor {

	/**
	 * Abstract QoS-Classes for the Accuracy of the position. Implementation
	 * depends on the {@link EnergyComponent}
	 * 
	 * @author Bjoern Richerzhagen
	 * @version 1.0, 26.02.2012
	 */
	@Deprecated
	public static enum PositionAccuracy {
		HIGH, MEDIUM, LOW
	}

	/**
	 * This uses an {@link EnergyComponent} for position retrieving (ie a GPS).
	 * A call might trigger energy consumption and the result will in most cases
	 * not be 100% accurate. Use this in your application if you want to add
	 * another layer of realism.
	 * 
	 * @param accuracy
	 *            a QoS-Class for the accuracy
	 * @return An estimate of the current position
	 */
	@Deprecated
	public Position getPosition(PositionAccuracy accuracy);

	/**
	 * Returns the Topology-Object that provides access to {@link TopologyView}s
	 * 
	 * @return
	 */
	public Topology getTopology();

}
