/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.api.topology.movement;

import de.tud.kom.p2psim.api.topology.TopologyComponent;

/**
 * A movement model for a group of {@link TopologyComponent}s. The movement
 * model interacts with the topology to move hosts and to react to obstacles, if
 * needed.
 * 
 * @author Bjoern Richerzhagen
 * @version 1.0, 21.02.2012
 */
public interface MovementModel {

	/**
	 * Add a {@link MovementSupported} component to this movement model
	 * 
	 * @param comp
	 */
	public void addComponent(MovementSupported comp);

	/**
	 * Adds a {@link MovementListener} to this {@link MovementModel}
	 * 
	 * @param listener
	 */
	public void addMovementListener(MovementListener listener);

	/**
	 * removes a listener
	 * 
	 * @param listener
	 */
	public void removeMovementListener(MovementListener listener);

	/**
	 * If you want to trigger the movement periodically, set this to a time
	 * 
	 * @param time
	 */
	public void setTimeBetweenMoveOperations(long time);

}
