/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.api.topology.movement;

import de.tud.kom.p2psim.impl.topology.PositionVector;

/**
 * This is the Listener for a Position. The method reached Position will be
 * called, if the given Position and radius is reached. This mean, the position
 * of the host is in the radius of the specified Position.
 * 
 * @author Christoph Muenker
 * @version 1.0, 28.06.2013
 */
public interface PositionListener {

	/**
	 * Specified the Position, which is listen
	 * 
	 * @return The position, to which is listen
	 */
	public PositionVector getPosition();

	/**
	 * Specified the radius, for to the position, at when the listener should be
	 * called.
	 * 
	 * @return the radius
	 */
	public double getRadius();

	/**
	 * Will be called, if the specified position is reached.
	 */
	public void reachedPosition();
}
