/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.linklayer.mac;

import de.tud.kom.p2psim.api.common.SimHost;
import de.tud.kom.p2psim.api.linklayer.mac.MacAddress;
import de.tud.kom.p2psim.api.linklayer.mac.MacLayer;
import de.tud.kom.p2psim.api.linklayer.mac.PhyType;
import de.tud.kom.p2psim.api.network.BandwidthImpl;
import de.tudarmstadt.maki.simonstrator.api.Message;

/**
 * This MAC provides an outgoing queue and schedules messages in a shared
 * Medium. It is ensured that no MAC sends while another MAC within range is
 * sending. Of course, this only makes sense in a Broadcast-Medium.
 * 
 * @author Bjoern Richerzhagen
 * @version 1.0, 27.03.2012
 */
public class CollisionAwareMac extends EnqueuingMac {

	/**
	 * Create a new MAC for a broadcast-medium that avoids collisions
	 * 
	 * @param ownMacAddress
	 * @param phy
	 * @param maxQueueLength
	 * @param maxTimeInQueue
	 * @param maxRetransmissions
	 * @param bandwidth
	 *            the maximum BW of this MAC (i.e. the BW that would be achieved
	 *            if messages are not queued)
	 */
	public CollisionAwareMac(SimHost host, MacAddress ownMacAddress,
			PhyType phy,
			int maxQueueLength, long maxTimeInQueue, int maxRetransmissions,
			BandwidthImpl bandwidth) {
		super(host, ownMacAddress, phy, maxQueueLength, maxTimeInQueue,
				maxRetransmissions, bandwidth);
		if (!phy.isBroadcastMedium()) {
			throw new AssertionError(
					"The CollisionAwareMac requires a Broadcast-PHY such as WIFI or BLUETOOTH");
		}
	}

	/*
	 * 
	 * TODO
	 */

	/**
	 * The EventInfo-Object that is used by this MAC to track blocking of MACs
	 * 
	 * @author Bjoern Richerzhagen
	 * @version 1.0, 27.03.2012
	 */
	protected class CollisionAwareMacEventInfo extends
			DefaultMacEventInformation {

		public CollisionAwareMacEventInfo(Message msg, EnqueuingMac sender,
				MacAddress receiver, long timeInQueue) {
			super(msg, sender.getMacAddress(), receiver, timeInQueue);
		}

		@Override
		public void arrivedAt(MacLayer receiver, boolean wasDropped) {
			super.arrivedAt(receiver, wasDropped);
		}

	}


}
