/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.topology.views.visualization.ui;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import com.google.common.collect.Maps;

import de.tud.kom.p2psim.impl.topology.views.visualization.ComponentVisManager;
import de.tud.kom.p2psim.impl.topology.views.visualization.ComponentVisManager.VisInfo;
import de.tud.kom.p2psim.impl.topology.views.visualization.ComponentVisManager.VisualizationListener;

/**
 * This component displays check boxes for each UI component that is added
 * through the ComponenVisManager. It enables the user to toggle the visibility
 * of each visualization component.
 * 
 * @author Fabio Zöllner
 * @version 1.0, 12.07.2012
 */
public class ComponentToggler extends JPanel {
	private Map<String, JCheckBox> componentNameToCheckBoxMap = Maps
			.newConcurrentMap();

	private ComponentVisManager visManager;

	private JPanel spacer = new JPanel();

	public ComponentToggler(ComponentVisManager visManager) {
		this.visManager = visManager;

		setBorder(BorderFactory.createTitledBorder("Layer"));
		setLayout(new GridBagLayout());
		setPreferredSize(new Dimension(150, 500));

		visManager.addVisualizationListener(new VisualizationListener() {
			@Override
			public void visualizationRemoved(VisInfo visInfo) {
				ComponentToggler.this.remove(componentNameToCheckBoxMap
						.remove(visInfo.getName()));
				componentNameToCheckBoxMap.remove(visInfo.getName());
			}

			@Override
			public void visualizationAdded(VisInfo visInfo) {
				addCheckBox(visInfo);
			}
		});

		initCheckBoxes();
	}

	private void initCheckBoxes() {
		for (VisInfo visInfo : visManager.getVisualizations()) {
			addCheckBox(visInfo);
		}
	}

	private void addCheckBox(final VisInfo visInfo) {

		if (componentNameToCheckBoxMap.get(visInfo.getName()) != null)
			return;

		JCheckBox checkBox = new JCheckBox(visInfo.getName());
		checkBox.setSelected(visInfo.isActiveByDefault());
		checkBox.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				visManager.toggleComponent(visInfo.getName());
			}
		});
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.fill = GridBagConstraints.NONE;
		constraints.gridx = 0;
		constraints.anchor = GridBagConstraints.FIRST_LINE_START;
		constraints.weighty = 0;
		constraints.ipady = 0;

		if (visInfo.isShowInList())
			add(checkBox, constraints);

		componentNameToCheckBoxMap.put(visInfo.getName(), checkBox);

		for (JComponent comp : visInfo.getComponents()) {
			if (comp instanceof ComponentOptions) {
				ComponentOptions co = (ComponentOptions) comp;
				for (JCheckBox box : co.getCheckBoxes()) {
					JPanel panel = new JPanel();
					panel.add(new JLabel("\u2192"));
					panel.add(box);
					add(panel, constraints);
				}
			}
		}

		remove(spacer);
		constraints.weighty = 1;
		add(spacer, constraints);

		this.validate();
	}
}
