/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.topology.views.visualization.ui;

import java.awt.FlowLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.List;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import com.google.common.collect.Lists;

class Selector {
	private String name;
	private Object[] values;
	private JComboBox selector;
	private JLabel label;
	private JPanel panel;
	private List<SelectionListener> selectionListeners = Lists.newArrayList();
	private Object deselectedItem = "";
	
	public Selector(final String name, Object... values) {
		this.name = name;
		this.values = values;

		buildSelector();
		
		selector.addItemListener(new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent event) {
				if (event.getStateChange() == ItemEvent.DESELECTED) {
					deselectedItem = event.getItem();
				} else {
					for (SelectionListener l : selectionListeners) {
						//System.err.println(event.getItem().getClass().getSimpleName());
						l.selectionChanged(name, deselectedItem, event.getItem());
					}
				}
			}
		});
	}
	
	public void removeSelectorListener(SelectionListener listener) {
		selectionListeners.remove(listener);
	}

	public void addSelectorListener(SelectionListener listener) {
		selectionListeners.add(listener);
	}

	private void buildSelector() {
		selector = new JComboBox(values);
		
		label = new JLabel(name + ": ");
		
		panel = new JPanel();
		panel.setLayout(new FlowLayout());
		
		panel.add(label);
		panel.add(selector);
	}

	public String getName() {
		return name;
	}
	
	public Object[] getValues() {
		return values;
	}
	
	public JPanel getSelector() {
		return panel;
	}

	public void setValues(Object... values) {
		this.values = values;
		// Notify selection listeners?
	}
}
