/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.topology.views.visualization.world;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.List;
import java.util.Vector;

import javax.swing.JComponent;

import de.tud.kom.p2psim.api.topology.TopologyComponent;
import de.tud.kom.p2psim.impl.topology.PositionVector;
import de.tud.kom.p2psim.impl.topology.views.VisualizationTopologyView;
import de.tud.kom.p2psim.impl.topology.views.VisualizationTopologyView.VisualizationInjector;

/**
 * Visualization of a component
 * 
 * @author Bjoern Richerzhagen
 * @version 1.0, 19.03.2012
 */
public class TopologyComponentVis extends JComponent {

	/**
	 * 
	 */
	protected final VisualizationTopologyView visualizationTopologyView;

	protected final int COMPONENT_PADDING = 15;

	protected final int COMPONENT_RADIUS = 3;

	protected final Color COLOR_OFFLINE = new Color(100, 100, 100, 50);

	protected Color color_online = new Color(255, 0, 255, 255);

	private TopologyComponent tComp;

	private PositionVector posVec;
	
	protected final long hostid;

	private List<LinkComponentVis> myLinks = new Vector<LinkComponentVis>();

	protected boolean active = false;

	protected boolean clicked = false;

	public TopologyComponentVis(VisualizationTopologyView visualizationTopologyView, TopologyComponent comp) {
		this.visualizationTopologyView = visualizationTopologyView;
		tComp = comp;
		posVec = comp.getRealPosition();
		hostid = comp.getHost().getHostId();
		
		String peerColor = comp.getHost().getProperties().getProperty("peerColor");
		if (peerColor != null) {
			int cInt = Integer.parseInt(peerColor, 16);
			color_online = new Color(cInt);
		}
		
		Point position = comp.getRealPosition().asPoint();
		setBounds((int) (position.x / this.visualizationTopologyView.getScale() - COMPONENT_PADDING),
				(int) (position.y / this.visualizationTopologyView.getScale() - COMPONENT_PADDING),
				2 * COMPONENT_PADDING,
				2 * COMPONENT_PADDING);
		setOpaque(false);
		this.addMouseListener(new MouseAdapter() {

			/**
			 * Stores the mouse position, if the mouse button is pressed
			 */
			@Override
			public void mousePressed(MouseEvent e) {
				clicked = !clicked;
				TopologyComponentVis.this.visualizationTopologyView
						.notifyInteractionListenersOnClick(hostid, clicked);
				VisualizationInjector.getWorldPanel().invalidate();
			}

			@Override
			public void mouseEntered(MouseEvent e) {
				if (!active)
					toggleActive();
			}

			@Override
			public void mouseExited(MouseEvent e) {
				if (active && !clicked) {
					toggleActive();
				}
			}

		});
	}

	protected void paintComponent(Graphics g) {
		super.paintComponent(g);
		setLocation((int) ((posVec.getX() - COMPONENT_PADDING) / this.visualizationTopologyView.getScale()),
				(int) ((posVec.getY() - COMPONENT_PADDING) / this.visualizationTopologyView.getScale()));
		Graphics2D g2 = (Graphics2D) g;
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);

		if (clicked) {
			g2.setColor(Color.MAGENTA);
		} else {
			g2.setColor(Color.DARK_GRAY);
		}

		if (!tComp.getHost().getNetworkComponent().getNetworkInterfaces()
				.iterator().next().isUp()) {
			g2.setColor(COLOR_OFFLINE);
		}

		g2.fillOval(COMPONENT_PADDING - COMPONENT_RADIUS, COMPONENT_PADDING
				- COMPONENT_RADIUS, COMPONENT_RADIUS * 2,
				COMPONENT_RADIUS * 2);


		g2.setColor(Color.BLACK);
		g2.setFont(VisualizationTopologyView.FONT_TINY);
		g2.drawString("ID: " + hostid, 0, COMPONENT_PADDING
				- COMPONENT_RADIUS);
		g2.setColor(Color.BLACK);
	}

	protected void toggleActive() {
		return;
		/*
		active = !active;
		if (active) {

			TopologyView tView = tComp.getTopology().getTopologyView(this.visualizationTopologyView.PHY);
			if (tView == null) {
				active = !active;
				return;
			}
			Simulator.getScheduler().pause();
			List<MacAddress> neighbors = tView.getNeighbors(mac
					.getMacAddress());
			for (MacAddress mAddr : neighbors) {
				Link l = tView.getLinkBetween(mac.getMacAddress(), mAddr);
				LinkComponentVis lVis = new LinkComponentVis(this.visualizationTopologyView, l, this,
						this.visualizationTopologyView.getComponentVis(mAddr));
				myLinks.add(lVis);
				VisualizationInjector.getWorldPanel().add(lVis);
			}
			Simulator.getScheduler().unpause();
		} else {
			for (LinkComponentVis lVis : myLinks) {
				VisualizationInjector.getWorldPanel().remove(lVis);
			}
		}
		VisualizationInjector.getWorldPanel().invalidate();
		*/
	}

	protected PositionVector getPosition() {
		return posVec;
	}

}